function [ Ys, W, I ] = MMI_matrix_param(Xs,k,Jp,d,W0)
%
% Maximize mutual information between projected space Y and the
% class label C.
%
% Inputs:
%   Xs: original feature patterns of size [ D, N ]
%   k: pattern labels of size [ N, 1 ]
%   Jp: 
%   d: desired reduced dimension (must satisfy 1 <= d <= D)
%   W0: initial guess at the projection matrix of size [ D, d ]
% Outputs:
%   W: final projection matrix of size [ D, d ]
% 
% Written by:
% -- 
% John L. Weatherwax                2005-10-30
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

MAX_OUTER_LOOPS=5; 
MAX_INNER_LOOPS=3; 
FRAC_DELTA_I_MIN=1E-3; 

%
% Code begins:

[ N, D ] = size(Xs); 
if( d>D ) error( 'Input value of d is too large' ); end

W = W0; 
Ys = Xs * W;
sgma = halfTwoFarthestPoints( Ys ); %sgma=5;
no=0; 
fprintf('>s: no=%3d: sgma=%g\n',no,sgma);

% Do one step: 
[ dICdY, dITdY, IC, IT ] = computeInformationGradients(Ys,k,sgma,Jp);
dIdY = dITdY; I = IT; I_old = I; 
Wud = dMIdW(dIdY,Xs);

while( 1 ) % sgma reduction/refinement loop
  ni=0; 
  while( 1 ) % mutual information reduction loop
    
    % Plot this projection: 
    if( 0 ) 
    cs = { 'yo', 'g*', 'bd', 'rs', 'bx', 'm+' }; 
    figure; 
    for i=1:length(unique(k))
      inds=find(k==i);
      %plot( -Ys(inds,1), +Ys(inds,2), cs{i} ); 
      plot( +Ys(inds,1), -Ys(inds,2), cs{i} ); 
      %plot( -Ys(inds,1), -Ys(inds,2), cs{i} ); 
      if( i==1 ) hold on; end
    end
    axis( [ -3.5 3.5 -3.5 3.5 ] ); grid on; 
    title( ['Projected Data; I=',num2str(I)] ); drawnow; 
    end
    
    % Perform very naive line search to find an "optimal" eta:
    % --first bracket our maximum by testing a great number of hypothesis:
    etaRng = 10.^[ -1:10 ]; 
    for ei=1:length(etaRng)
      Wp = W + etaRng(ei) * Wud; 
      Q = gram(Wp); Wp = Q; %clear Q; 
      Ys = Xs * Wp;
      [ VCY2, VC2Y2, VCY ] = computeInformationPotentials(Ys,k,Jp,sgma); 
      IT = VCY2 + VC2Y2 - 2*VCY;
      %fprintf( '  %5d: ETA=%10g; IT=%20g', ei, etaRng(ei), IT ); 
      %if( ei~=length(etaRng) ) fprintf('\n'); end
      Is(ei) = IT; 
    end
    % select the best eta: 
    [dum,ii]=max(Is); fprintf('          i_opt=%d\n',ii); 
    W = W + etaRng(ii) * Wud; 
    Q = gram(W); W = Q; %clear Q; 
    Ys = Xs * W;
    
    % Do next set of gradients: 
    [ dICdY, dITdY, IC, IT ] = computeInformationGradients(Ys,k,sgma,Jp);
    dIdY = dITdY; I = IT; 
    Wud = dMIdW(dIdY,Xs);
    
    if( 0 ) 
    cs = { 'ko', 'k*', 'kd', 'ks', 'kx', 'k+' }; 
    for i=1:length(unique(k))
      inds=find(k==i);
      plot( -Ys(inds,1), Ys(inds,2), cs{i} ); 
    end
    axis( [ -3.5 3.5 -3.5 3.5 ] ); grid on; 
    title( 'Projected Data' );    
    end
    
    %pause;
    %keyboard; 
    
    fracDeltaI=(I-I_old)/I_old; I_old=I;
    fprintf( '  >f: ni=%3d: fracDeltaI = %g\n', ni, fracDeltaI ); 
    
    ni=ni+1; 
    if( ni > MAX_INNER_LOOPS | fracDeltaI < FRAC_DELTA_I_MIN ) break; end
  end
  sgma=sgma/2; 
  fprintf( '>f: no=%3d: sgma=%g\n', no, sgma );
  no=no+1; 
  if( no > MAX_OUTER_LOOPS | sgma < halfInClassTwoFarthestPoints( Ys, k ) ) break; end
end
